// ============================================================================
//  Project:	Course "Object-Oriented Programming (in Java)"
//  Version:	Winter 2014/15, University of Halle
//  Module:	Map.java
//  Purpose:	Homework 11: Small Computer Game (Rogue/Nethack/Larn-Style)
//  LastChange:	23.01.2015
//  Language:	Java
//  Authors:	Stefan Brass
//  Email:	brass@informatik.uni-halle.de
//  Address:	Universitaet Halle, Inst. f. Informatik, D-06099 Halle, Germany
//  Copying:	You may do what you want with this, but don't make me
//		responsible. This code probably contains bugs.
// ============================================================================


// ============================================================================
// Klasse fuer Spielfelder:
// ============================================================================

abstract class Field {

	// Ist dieses Feld schon bekannt und soll angezeigt werden?
	private boolean known;

	// Konstruktor:
	Field() {
		known = false;
	}

	// Ist dieses Feld schon bekannt und soll angezeigt werden?
	boolean isKnown() {
		return known;
	}

	// Dieses Feld wurde gesehen:
	void makeKnown() {
		known = true;
	}

	// Kann man das Feld verlassen? Dann null liefern. Sonst Begruendung.
	abstract String cantLeave(Player p);

	// Kann man das Feld betreten? Falls ja: null. Falls nein: Begruendung.
	abstract String cantEnter(Player p);

	// Passiert etwas mit dem Spieler, wenn er das Feld verlaesst?
	abstract String leave(Player p);

	// Passiert etwas mit dem Spieler, wenn er das Feld betritt?
	// Ggf. Erklaerung liefern, Player-Objekt veraendern. Sonst null.
	abstract String enter(Player p);

	// Farbe des Untergrundes (schwarz falls isWall, weiss sonst):
	abstract boolean isWall();

	// Zeichen (falls gewuenscht, sonst ' '):
	char symbol() {
		return ' ';
	}
}

// ============================================================================
// Klasse fuer nicht betretbare Spielfelder ("Wall"):
// ============================================================================

class Wall extends Field {
	String cantLeave(Player p) {
		return "Du bist eingemauert.";
	}

	String cantEnter(Player p) {
		return "Da ist eine Wand.";
	}

	String enter(Player p) {
		return null;
	}

	String leave(Player p) {
		return null;
	}

	boolean isWall() {
		return true;
	}

	// char symbol() {
	//	return '#';
	// }
}

// ============================================================================
// Klasse fuer normale, betretbare Spielfelder ("Path"):
// ============================================================================

class Path extends Field {
	String cantLeave(Player p) {
		return null;
	}

	String cantEnter(Player p) {
		return null;
	}

	String enter(Player p) {
		return null;
	}

	String leave(Player p) {
		return null;
	}

	boolean isWall() {
		return false;
	}

	// char symbol() {
	//	return '.';
	// }
}


// ============================================================================
// Klasse fuer die Karte (gesamtes "Spielbrett"):
// ============================================================================

class Map {
	// Groesse des Spielfeldes:
	private int sizeX;
	private int sizeY;

	// Matrix von Feldern:
	private Field[][] map;

	// Konstruktor:
	Map(int sizeX, int sizeY) {
		this.sizeX = sizeX;
		this.sizeY = sizeY;
		map = new Field[sizeX][sizeY];
	}

	// Anzahl Felder in X-Richtung liefern:
	int getSizeX() {
		return this.sizeX;
	}

	// Anzahl Felder in Y-Richtung liefern:
	int getSizeY() {
		return this.sizeY;
	}

	// Spielfeld initialisieren:
	void makeField(int x, int y, char type) {
		if(x < 0 || x >= this.sizeX)
			throw new IllegalArgumentException(
					"X-Wert ungueltig: " + x);
		if(y < 0 || y >= this.sizeY)
			throw new IllegalArgumentException(
					"Y-Wert ungueltig: " + y);
		switch(type) {
			case '#':
				map[x][y] = new Wall();
				break;
			case '.':
			case '@':
			case '<':
				map[x][y] = new Path();
				break;
			default:
				throw new IllegalArgumentException(
					"Unbekannter Feld-Typ: " + type);
		}
	}

	// Spielfeld abfragen:
	Field getField(int x, int y) {
		if(x < 0 || x >= this.sizeX)
			throw new IllegalArgumentException(
					"X-Wert ungueltig: " + x);
		if(y < 0 || y >= this.sizeY)
			throw new IllegalArgumentException(
					"Y-Wert ungueltig: " + y);
		return map[x][y];
	}
}

